package me.rainsay.testdemo;
import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static com.github.tomakehurst.wiremock.core.WireMockConfiguration.wireMockConfig;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import org.junit.Before;
import org.junit.ClassRule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.TestConfiguration;
import org.springframework.cloud.netflix.ribbon.StaticServerList;
import org.springframework.context.annotation.Bean;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import com.github.tomakehurst.wiremock.junit.WireMockRule;
import com.google.common.net.MediaType;
import com.netflix.loadbalancer.Server;
import com.netflix.loadbalancer.ServerList;

import me.rainsay.testdemo.feign.BookClient;

@RunWith(SpringRunner.class)
@SpringBootTest(properties = {
		"feign.hystrix.enabled=true",
		"eureka.client.enabled=false"
})
@ContextConfiguration(classes = {
		SomeTest.LocalRibbonClientConfiguration.class
})
public class SomeTest {
	@ClassRule
	public static WireMockRule wireMock = new WireMockRule(wireMockConfig().dynamicPort());
	
	@Autowired
	BookClient bookClient;
	
	@Before
	public void setup() {
		stubFor(get(urlEqualTo("/book"))
				.willReturn(aResponse()
						.withStatus(200)
						.withHeader("Content-Type", MediaType.PLAIN_TEXT_UTF_8.toString())
						.withBody("Hello World"))
				);
	}
	
	@Test
	public void testHello() {
		String hello = bookClient.hello();
		assertNotNull("shoud not be null",hello);
		assertEquals("Hello World", hello);
	}
	
	@Test
	public void testHelloFallback() {
		stubFor(get(urlEqualTo("/book"))
                .willReturn(aResponse().withFixedDelay(60000)));
		String hello = bookClient.hello();
		assertNotNull(hello);
		assertEquals("No hello", hello);
	}
	
	@TestConfiguration
	public static class LocalRibbonClientConfiguration {
		@Bean
		public ServerList<Server> ribbonServerList() {
			return new StaticServerList<>(new Server("localhost", wireMock.port()));
		}
	}
}
